/* Este programa exemplo (main.c) resolve um sistema linear do tipo
 * Ax = b onde A eh uma matriz de coeficientes, b eh o vetor numerico e 
 * x eh o vetor de incognitas.
 */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "nrutil.h"

#define N 5

int main(){
    int i, j, k;

    /* Declaracoes das matrizes para o Numerical Recipes : */

    double **a  = dmatrix(1, N, 1, N);
    double **aa = dmatrix(1, N, 1, N);
    double **p  = dmatrix(1, N, 1, N);
    double **b  = dmatrix(1, N, 1, 1);		/* a funcao gaussj espera que
						   b seja uma matriz... */
    double *x   = dvector(1, N);

    /* Criando A e b:
     * Note que o Numerical Recipes convenciona os índices comecando em 1.
     */
    for (i = 1; i <= N; i++){			/* i = linha    */
	for (j = 1; j <= N; j++) {		/* j = coluna */
	    a[i][j] = 1.0 / (i+j);
	    /* Salve uma copia de a, para uso posterior. */
	    aa[i][j] = a[i][j];
	}
    }

    for (i = 1; i <= N; i++){
	/* Lembre-se b eh uma matriz de uma coluna */
	b[i][1] = 1;
    }

    /* imprimindo a matriz A e b iniciais */

    printf("inicial A e b:\n");

    for (i = 1; i <= N; i++) {
	printf("    ");
	for (j = 1; j <= N; j++){ 
		printf("%9.2e  ", a[i][j]);
	}
	printf("    %9.2e\n", b[i][1]);
    }

    /* Usando a rotina gaussj: A passa a ser a inversa e b a solucao */

    gaussj(a, N, b, 1);

    /* Imprime a matriz A final e b. */

    printf("final A(inv) e b' (sol):\n");

    for (i = 1; i <= N; i++) {
	printf("    ");
	for (j = 1; j <= N; j++){ 
		printf("%9.2e  ", a[i][j]);
	}
	printf("    %9.2e\n", b[i][1]);
    }

    /* Para certificar que 'a' e 'aa' sao inversas, fazemos o produto das 
     * duas para ver se obtenos a matriz identidade */

    for (i = 1; i <= N; i++){
	for (j = 1; j <= N; j++) {
	    p[i][j] = 0;
	    for (k = 1; k <= N; k++){
		p[i][j] += a[i][k]*aa[k][j];
	    }
	}
    }

    for (i = 1; i <= N; i++) {
	x[i] = 0;
	for (j = 1; j <= N; j++){
	    x[i] += aa[i][j]*b[j][1];
	}
    }

    /* Imprime a matriz P e x. */

    printf("produto de A * Ainv, A * b':\n");

    for (i = 1; i <= N; i++) {
	printf("    ");
	for (j = 1; j <= N; j++){
		printf("%9.5f  ", p[i][j]);
	}
	printf("    %9.2e\n", x[i]);
    }
    exit(0);
}

